/*==================================================================
 *      File :          table.c
 *      Package:        Xmeasure
 * 
 *      Author :        Aloke Gupta.
 *
 *  (C) Copyright 1992, Aloke Gupta.
 *  All rights granted to University of Illinois Board of Regents.
 *==================================================================*/

#include "perf.h"
#include "tests.h"

TestRequest testrequest[] = {
{  0, "Undefined Request",       WIN, FALSE,
   i_NoOperation,            NoOperation,            c_NoOperation,
   { {INVALID,  0},{INVALID,  0},{INVALID,  0},{INVALID,  0},{INVALID,  0}, } },

{  1, "CreateWindow",            WIN, FALSE,
   i_CreateWindow,           CreateWindow,           c_CreateWindow,           
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{  2, "ChangeWindowAttributes",  WIN, FALSE,
   i_ChangeWindowAttributes, ChangeWindowAttributes, c_ChangeWindowAttributes, 
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{  3, "GetWindowAttributes",     WIN, FALSE,
   i_GetWindowAttributes,    GetWindowAttributes,    c_GetWindowAttributes,    
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{  4, "DestroyWindow",           WIN, FALSE,
   i_DestroyWindow,          DestroyWindow,          c_DestroyWindow,          
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{  5, "DestroySubwindows",       WIN, FALSE,
   i_DestroySubwindows,      DestroySubwindows,      c_DestroySubwindows,      
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{  6, "ChangeSaveSet",           WIN, FALSE,
   i_ChangeSaveSet,          ChangeSaveSet,          c_ChangeSaveSet,          
   { {INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{  7, "ReparentWindow",          WIN, FALSE,
   i_ReparentWindow,         ReparentWindow,         c_ReparentWindow,         
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{  8, "MapWindow",               WIN, FALSE,
   i_MapWindow,              MapWindow,              c_MapWindow,              
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{  9, "MapSubwindows",           WIN, FALSE,
   i_MapSubwindows,          MapSubwindows,          c_MapSubwindows,          
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 10, "UnmapWindow",             WIN, FALSE,
   i_UnmapWindow,            UnmapWindow,            c_UnmapWindow,            
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 11, "UnmapSubwindows",         WIN, FALSE,
   i_UnmapSubwindows,        UnmapSubwindows,        c_UnmapSubwindows,        
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 12, "ConfigureWindow",         WIN, FALSE,
   i_ConfigureWindow,        ConfigureWindow,        c_ConfigureWindow,        
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 13, "CirculateWindow",         WIN, FALSE,
   i_CirculateWindow,        CirculateWindow,        c_CirculateWindow,        
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 14, "GetGeometry",             WIN, FALSE,
   i_GetGeometry,            GetGeometry,            c_GetGeometry,            
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 15, "QueryTree",               WIN, FALSE,
   i_QueryTree,              QueryTree,              c_QueryTree,              
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 16, "InternAtom",              WIN, FALSE,
   i_InternAtom,             InternAtom,             c_InternAtom,             
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 17, "GetAtomName",             WIN, FALSE,
   i_GetAtomName,            GetAtomName,            c_GetAtomName,            
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 18, "ChangeProperty",          WIN, FALSE,
   i_ChangeProperty,         ChangeProperty,         c_ChangeProperty,         
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 19, "DeleteProperty",          WIN, FALSE,
   i_DeleteProperty,         DeleteProperty,         c_DeleteProperty,         
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 20, "GetProperty",             WIN, FALSE,
   i_GetProperty,            GetProperty,            c_GetProperty,            
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 21, "ListProperties",          WIN, FALSE,
   i_ListProperties,         ListProperties,         c_ListProperties,         
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 22, "SetSelectionOwner",       WIN, FALSE,
   i_SetSelectionOwner,      SetSelectionOwner,      c_SetSelectionOwner,      
   { {INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 23, "GetSelectionOwner",       WIN, FALSE,
   i_GetSelectionOwner,      GetSelectionOwner,      c_GetSelectionOwner,      
   { {INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 24, "ConvertSelection",        WIN, FALSE,
   i_ConvertSelection,       ConvertSelection,       c_ConvertSelection,       
   { {INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 25, "SendEvent",               WIN, FALSE,
   i_SendEvent,              SendEvent,              c_SendEvent,              
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 26, "GrabPointer",             WIN, FALSE,
   i_GrabPointer,            GrabPointer,            c_GrabPointer,            
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 27, "UngrabPointer",           WIN, FALSE,
   i_UngrabPointer,          UngrabPointer,          c_UngrabPointer,          
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 28, "GrabButton",              WIN, FALSE,
   i_GrabButton,             GrabButton,             c_GrabButton,             
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 29, "UngrabButton",            WIN, FALSE,
   i_UngrabButton,           UngrabButton,           c_UngrabButton,           
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 30, "ChangeActivePointerGrab", WIN, FALSE,
   i_ChangeActivePointerGrab,ChangeActivePointerGrab,c_ChangeActivePointerGrab,
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 31, "GrabKeyboard",            WIN, FALSE,
   i_GrabKeyboard,           GrabKeyboard,           c_GrabKeyboard,           
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 32, "UngrabKeyboard",          WIN, FALSE,
   i_UngrabKeyboard,         UngrabKeyboard,         c_UngrabKeyboard,         
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 33, "GrabKey",                 WIN, FALSE,
   i_GrabKey,                GrabKey,                c_GrabKey,                
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 34, "UngrabKey",               WIN, FALSE,
   i_UngrabKey,              UngrabKey,              c_UngrabKey,              
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 35, "AllowEvents",             WIN, FALSE,
   i_AllowEvents,            AllowEvents,            c_AllowEvents,            
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 36, "GrabServer",              WIN, FALSE,
   i_GrabServer,             GrabServer,             c_GrabServer,             
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 37, "UngrabServer",            WIN, FALSE,
   i_UngrabServer,           UngrabServer,           c_UngrabServer,           
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 38, "QueryPointer",            WIN, FALSE,
   i_QueryPointer,           QueryPointer,           c_QueryPointer,           
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 39, "GetMotionEvents",         WIN, FALSE,
   i_GetMotionEvents,        GetMotionEvents,        c_GetMotionEvents,        
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 40, "TranslateCoordinates",    WIN, FALSE,
   i_TranslateCoordinates,   TranslateCoordinates,   c_TranslateCoordinates,   
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 41, "WarpPointer",             WIN, FALSE,
   i_WarpPointer,            WarpPointer,            c_WarpPointer,            
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 42, "SetInputFocus",           WIN, FALSE,
   i_SetInputFocus,          SetInputFocus,          c_SetInputFocus,          
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 43, "GetInputFocus",           WIN, FALSE,
   i_GetInputFocus,          GetInputFocus,          c_GetInputFocus,          
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 44, "QueryKeymap",             WIN, FALSE,
   i_QueryKeymap,            QueryKeymap,            c_QueryKeymap,            
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 45, "OpenFont",                WIN, FALSE,
   i_OpenFont,               OpenFont,               c_OpenFont,               
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 46, "CloseFont",               WIN, FALSE,
   i_CloseFont,              CloseFont,              c_CloseFont,              
   { {INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 47, "QueryFont",               WIN, FALSE,
   i_QueryFont,              QueryFont,              c_QueryFont,              
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 48, "QueryTextExtents",        WIN, FALSE,
   i_QueryTextExtents,       QueryTextExtents,       c_QueryTextExtents,       
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 49, "ListFonts",               WIN, FALSE,
   i_ListFonts,              ListFonts,              c_ListFonts,              
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 50, "ListFontsWithInfo",       WIN, FALSE,
   i_ListFontsWithInfo,      ListFontsWithInfo,      c_ListFontsWithInfo,      
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 51, "SetFontPath",             WIN, FALSE,
   i_SetFontPath,            SetFontPath,            c_SetFontPath,            
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 52, "GetFontPath",             WIN, FALSE,
   i_GetFontPath,            GetFontPath,            c_GetFontPath,            
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 53, "CreatePixmap",            WIN, FALSE,
   i_CreatePixmap,           CreatePixmap,           c_CreatePixmap,           
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 54, "FreePixmap",              WIN, FALSE,
   i_FreePixmap,             FreePixmap,             c_FreePixmap,             
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 55, "CreateGC",                WIN, FALSE,
   i_CreateGC,               CreateGC,               c_CreateGC,               
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 56, "ChangeGC",                WIN, FALSE,
   i_ChangeGC,               ChangeGC,               c_ChangeGC,               
   { {INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 57, "CopyGC",                  WIN, FALSE,
   i_CopyGC,                 CopyGC,                 c_CopyGC,                 
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 58, "SetDashes",               WIN, FALSE,
   i_SetDashes,              SetDashes,              c_SetDashes,              
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 59, "SetClipRectangles",       WIN, FALSE,
   i_SetClipRectangles,      SetClipRectangles,      c_SetClipRectangles,      
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 60, "FreeGC",                  WIN, FALSE,
   i_FreeGC,                 FreeGC,                 c_FreeGC,                 
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 61, "ClearArea",               WIN, FALSE,
   i_ClearArea,              ClearArea,              c_ClearArea,              
   { {10,10},{100, 100},{300, 300},{500, 500},{INVALID, 0}, } },

{ 62, "CopyArea",                WIN, FALSE,
   i_CopyArea,               CopyArea,               c_CopyArea,               
   { {10,10},{100, 100},{300, 300},{500, 500},{INVALID, 0}, } },

{ 63, "CopyPlane",               WIN, FALSE,
   i_CopyPlane,              CopyPlane,              c_CopyPlane,              
   { {10,10},{100, 100},{300, 300},{500, 500},{INVALID, 0}, } },

{ 64, "PolyPoint",               GFX, FALSE,
   i_PolyPoint,              PolyPoint,              c_PolyPoint,              
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 65, "PolyLine",                GFX, TRUE,
   i_PolyLine,               PolyLine,               c_PolyLine,               
   { {10,0},{100, 0},{300, 0},{500, 0},{INVALID, 0}, } },

{ 66, "PolySegment",             GFX, TRUE,
   i_PolySegment,            PolySegment,            c_PolySegment,            
   { {10,0},{100, 0},{300, 0},{500, 0},{INVALID, 0}, } },

{ 67, "PolyRectangle",           GFX, TRUE,
   i_PolyRectangle,          PolyRectangle,          c_PolyRectangle,          
   { {10, 10}, {100, 100},{300, 300},{500, 500},{INVALID, 0}, } },

{ 68, "PolyArc",                 GFX, TRUE,
   i_PolyArc,                PolyArc,                c_PolyArc,                
   { {10,10},{100, 100},{300, 300},{500, 500},{INVALID, 0}, } },

{ 69, "FillPoly",                GFX, FALSE,
   i_FillPoly,               FillPoly,               c_FillPoly,               
   { {INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 70, "PolyFillRectangle",       GFX, TRUE,
   i_PolyFillRectangle,      PolyFillRectangle,      c_PolyFillRectangle,      
   { {10, 10}, {100, 100},{300, 300},{500, 500},{INVALID, 0}, } },

{ 71, "PolyFillArc",             GFX, TRUE,
   i_PolyFillArc,            PolyFillArc,            c_PolyFillArc,            
   { {10,10},{100, 100},{300, 300},{500, 500},{INVALID, 0}, } },

{ 72, "PutImage",                WIN, TRUE,
   i_PutImage,               PutImage,               c_PutImage,               
   { {10, 10}, {100, 100},{300, 300},{500, 500},{INVALID, 0}, } },

{ 73, "GetImage",                WIN, TRUE,
   i_GetImage,               GetImage,               c_GetImage,               
   { {10,10},{100, 100},{300, 300},{500, 500},{INVALID, 0}, } },

{ 74, "PolyText8",               TXT, TRUE,
   i_PolyText8,              PolyText8,              c_PolyText8,              
   { {2, 0},{8, 0},{32, 0},{INVALID, 0},{INVALID, 0}, } },

{ 75, "PolyText16",              TXT, TRUE,
   i_PolyText16,             PolyText16,             c_PolyText16,             
   { {2, 0},{8, 0},{32, 0},{INVALID, 0},{INVALID, 0}, } },

{ 76, "ImageText8",              TXT, TRUE,
   i_ImageText8,             ImageText8,             c_ImageText8,             
   { {2, 0},{8, 0},{32, 0},{INVALID, 0},{INVALID, 0}, } },

{ 77, "ImageText16",             TXT, TRUE,
   i_ImageText16,            ImageText16,            c_ImageText16,            
   { {2, 0},{8, 0},{32, 0},{INVALID, 0},{INVALID, 0}, } },

{ 78, "CreateColormap",          WIN, FALSE,
   i_CreateColormap,         CreateColormap,         c_CreateColormap,         
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 79, "FreeColormap",            WIN, FALSE,
   i_FreeColormap,           FreeColormap,           c_FreeColormap,           
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 80, "CopyColormapAndFree",     WIN, FALSE,
   i_CopyColormapAndFree,    CopyColormapAndFree,    c_CopyColormapAndFree,    
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 81, "InstallColormap",         WIN, FALSE,
   i_InstallColormap,        InstallColormap,        c_InstallColormap,        
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 82, "UninstallColormap",       WIN, FALSE,
   i_UninstallColormap,      UninstallColormap,      c_UninstallColormap,      
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 83, "ListInstalledColormaps",  WIN, FALSE,
   i_ListInstalledColormaps, ListInstalledColormaps, c_ListInstalledColormaps, 
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 84, "AllocColor",              WIN, FALSE,
   i_AllocColor,             AllocColor,             c_AllocColor,             
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 85, "AllocNamedColor",         WIN, FALSE,
   i_AllocNamedColor,        AllocNamedColor,        c_AllocNamedColor,        
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 86, "AllocColorCells",         WIN, FALSE,
   i_AllocColorCells,        AllocColorCells,        c_AllocColorCells,        
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 87, "AllocColorPlanes",        WIN, FALSE,
   i_AllocColorPlanes,       AllocColorPlanes,       c_AllocColorPlanes,       
   { {INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 88, "FreeColors",              WIN, FALSE,
   i_FreeColors,             FreeColors,             c_FreeColors,             
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 89, "StoreColors",             WIN, FALSE,
   i_StoreColors,            StoreColors,            c_StoreColors,            
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 90, "StoreNamedColor",         WIN, FALSE,
   i_StoreNamedColor,        StoreNamedColor,        c_StoreNamedColor,        
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 91, "QueryColors",             WIN, FALSE,
   i_QueryColors,            QueryColors,            c_QueryColors,            
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 92, "LookupColor",             WIN, FALSE,
   i_LookupColor,            LookupColor,            c_LookupColor,            
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 93, "CreateCursor",            WIN, FALSE,
   i_CreateCursor,           CreateCursor,           c_CreateCursor,           
   { {INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 94, "CreateGlyphCursor",       WIN, FALSE,
   i_CreateGlyphCursor,      CreateGlyphCursor,      c_CreateGlyphCursor,      
   { {INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 95, "FreeCursor",              WIN, FALSE,
   i_FreeCursor,             FreeCursor,             c_FreeCursor,             
   { {INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 96, "RecolorCursor",           WIN, FALSE,
   i_RecolorCursor,          RecolorCursor,          c_RecolorCursor,          
   { {INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 97, "QueryBestSize",           WIN, FALSE,
   i_QueryBestSize,          QueryBestSize,          c_QueryBestSize,          
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 98, "QueryExtension",          WIN, FALSE,
   i_QueryExtension,         QueryExtension,         c_QueryExtension,         
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{ 99, "ListExtensions",          WIN, FALSE,
   i_ListExtensions,         ListExtensions,         c_ListExtensions,         
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{100, "ChangeKeyboardMapping",   WIN, FALSE,
   i_ChangeKeyboardMapping,  ChangeKeyboardMapping,  c_ChangeKeyboardMapping,  
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{101, "GetKeyboardMapping",      WIN, FALSE,
   i_GetKeyboardMapping,     GetKeyboardMapping,     c_GetKeyboardMapping,     
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{102, "ChangeKeyboardControl",   WIN, FALSE,
   i_ChangeKeyboardControl,  ChangeKeyboardControl,  c_ChangeKeyboardControl,  
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{103, "GetKeyboardControl",      WIN, FALSE,
   i_GetKeyboardControl,     GetKeyboardControl,     c_GetKeyboardControl,     
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{104, "Bell",                    WIN, FALSE,
   i_Bell,                   Bell,                   c_Bell,                   
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{105, "ChangePointerControl",    WIN, FALSE,
   i_ChangePointerControl,   ChangePointerControl,   c_ChangePointerControl,   
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{106, "GetPointerControl",       WIN, FALSE,
   i_GetPointerControl,      GetPointerControl,      c_GetPointerControl,      
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{107, "SetScreenSaver",          WIN, FALSE,
   i_SetScreenSaver,         SetScreenSaver,         c_SetScreenSaver,         
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{108, "GetScreenSaver",          WIN, FALSE,
   i_GetScreenSaver,         GetScreenSaver,         c_GetScreenSaver,         
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{109, "ChangeHosts",             WIN, FALSE,
   i_ChangeHosts,            ChangeHosts,            c_ChangeHosts,            
   { {INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{110, "ListHosts",               WIN, FALSE,
   i_ListHosts,              ListHosts,              c_ListHosts,              
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{111, "SetAccessControl",        WIN, FALSE,
   i_SetAccessControl,       SetAccessControl,       c_SetAccessControl,       
   { {INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{112, "SetCloseDownMode",        WIN, FALSE,
   i_SetCloseDownMode,       SetCloseDownMode,       c_SetCloseDownMode,       
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{113, "KillClient",              WIN, FALSE,
   i_KillClient,             KillClient,             c_KillClient,             
   { {INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{114, "RotateProperties",        WIN, TRUE,
   i_RotateProperties,       RotateProperties,       c_RotateProperties,       
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{115, "ForceScreenSaver",        WIN, FALSE,
   i_ForceScreenSaver,       ForceScreenSaver,       c_ForceScreenSaver,       
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{116, "SetPointerMapping",       WIN, FALSE,
   i_SetPointerMapping,      SetPointerMapping,      c_SetPointerMapping,      
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{117, "GetPointerMapping",       WIN, FALSE,
   i_GetPointerMapping,      GetPointerMapping,      c_GetPointerMapping,      
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{118, "SetModifierMapping",      WIN, FALSE,
   i_SetModifierMapping,     SetModifierMapping,     c_SetModifierMapping,     
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{119, "GetModifierMapping",      WIN, FALSE,
   i_GetModifierMapping,     GetModifierMapping,     c_GetModifierMapping,     
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{120, "Undefined Request",             WIN, FALSE,
   i_NoOperation,            NoOperation,            c_NoOperation,            
   { {INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{121, "Undefined Request",             WIN, FALSE,
   i_NoOperation,            NoOperation,            c_NoOperation,            
   { {INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{122, "Undefined Request",             WIN, FALSE,
   i_NoOperation,            NoOperation,            c_NoOperation,            
   { {INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{123, "Undefined Request",             WIN, FALSE,
   i_NoOperation,            NoOperation,            c_NoOperation,            
   { {INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{124, "Undefined Request",             WIN, FALSE,
   i_NoOperation,            NoOperation,            c_NoOperation,            
   { {INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{125, "Undefined Request",             WIN, FALSE,
   i_NoOperation,            NoOperation,            c_NoOperation,            
   { {INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{126, "Undefined Request",             WIN, FALSE,
   i_NoOperation,            NoOperation,            c_NoOperation,            
   { {INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },

{127, "NoOperation",             WIN, FALSE,
   i_NoOperation,            NoOperation,            c_NoOperation,            
   { {0, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0},{INVALID, 0}, } },
};
